@@ -414,11 +414,14 @@ class Job(Base):
             # handle pseudo files
             sin = self.stream_in
             sout = self.stream_out
+            cwd = self.working_directory if self.working_directory \
+                else os.getcwd()
             try:
                 self._process = subprocess.Popen(
                     cmd + [script_file.name],
                     stdin=sin,
                     stdout=sout,
+                    cwd=cwd
                 )
             except ValueError as err:
                 if str(err) == "redirected Stdin is pseudofile, "\
@@ -445,6 +448,8 @@ class Job(Base):
             # catch the errno 2 No such file or directory, which indicates the
             # interpreter is not available
             if err.errno == 2:
+                log.error("No interpreter found for script: %s", script_file,
+                          exc_info=True)
                 raise Exception("Interpreter %s not found!" % self.interpreter)
             raise err
 
