@@ -626,8 +626,19 @@ def submit_job(job, clean=False, force=False, save=True,
         if not log_file:
             continue
         parent = os.path.dirname(log_file)
+        if not parent:
+            continue
         if not os.path.exists(parent):
             os.makedirs(parent)
+
+    # Issue #37
+    # make sure working directories exist at submission time
+    if not os.path.exists(job.working_directory):
+        os.makedirs(job.working_directory)
+    for child in job.pipe_to:
+        if not os.path.exists(child.working_directory):
+            os.makedirs(child.working_directory)
+
     # submit the job
     cluster.submit(job)
     all_jobs = [job]
@@ -671,6 +682,20 @@ def run_job(job, save=False, profiler=False, submit_embedded=False):
     # createa the dispatcher graph
     dispatcher_nodes = jip.executils.create_dispatcher_graph(job)
     log.info("%s | Dispatch graph: %s", job, dispatcher_nodes)
+    # load job environment
+    env = job.env
+    if env is not None:
+        for k, v in env.iteritems():
+            log.info("Loading job environment %s:%s", k, v)
+            os.environ[k] = str(v)
+
+    # Issue #37
+    # make sure working directories exist at submission time
+    if not os.path.exists(job.working_directory):
+        os.makedirs(job.working_directory)
+    for child in job.pipe_to:
+        if not os.path.exists(child.working_directory):
+            os.makedirs(child.working_directory)
 
     for dispatcher_node in dispatcher_nodes:
         dispatcher_node.run(profiler=profiler)
@@ -866,17 +891,27 @@ def from_node(node, env=None, keep=False):
     job.keep_on_fail = keep
     job.tool_name = tool.name
     job.path = tool.path
-    job.working_directory = os.getcwd()
+    job.working_directory = os.path.normpath(
+        os.path.abspath(node._job.working_dir)
+    )
+    if not job.working_directory:
+        job.working_directory = os.getcwd()
+
     job.env = env if env is not None else create_job_env()
     if node._job is not None:
         node._job.apply(job)
     job.name = node.name
     job.pipeline = node._pipeline
 
+    # update job defaults after the profile is applied
+    if not job.threads:
+        job.threads = 1
+
     # store additional options
     job.configuration = node._tool.options
     job.configuration.make_absolute(job.working_directory)
     if node._additional_input_options:
+
         node_options = set(job.configuration.options)
         job.additional_options = set([])
         for a in node._additional_input_options:
@@ -1006,6 +1041,9 @@ def create_jobs(source, args=None, excludes=None, skip=None, keep=False,
         pipeline = p
     log.info("Jobs | Expanding pipeline with %d nodes", len(pipeline))
     pipeline.expand(validate=validate)
+    if profile is not None:
+        profile.apply_to_pipeline(pipeline)
+
     log.info("Jobs | Expanded pipeline has %d nodes", len(pipeline))
     if pipeline.excludes:
         if not excludes:
@@ -1024,14 +1062,13 @@ def create_jobs(source, args=None, excludes=None, skip=None, keep=False,
     # create all jobs. We keep the list for the order and
     # a dict to store the mapping from the node to teh job
     log.debug("Jobs | Creating job environment for %d nodes", len(pipeline))
-    env = create_job_env(profiler=profiler)
     nodes2jobs = {}
     jobs = []
     num_nodes = len(pipeline)
     for i, node in enumerate(pipeline.topological_order()):
         log.debug("Jobs | Creating job for %s (%d/%d)", node, i + 1, num_nodes)
         ## first create jobs
-        job = from_node(node, env=env, keep=keep)
+        job = from_node(node, keep=keep)
         log.debug("Jobs | Created job %s", job)
         jobs.append(job)
         nodes2jobs[node] = job
