@@ -6,9 +6,9 @@ from os import getenv
 import logging
 
 
-DEFAULT_FMT = "[%(levelname)s][%(asctime)s][%(name)s]: %(message)s"
-DEBUG_FMT = "[%(levelname)s][%(asctime)s][%(name)s]"\
-            "[%(module)s.%(funcName)s:%(lineno)d]: %(message)s"
+DEFAULT_FMT = "[%(levelname)6s][%(asctime)s][%(name)15s]: %(message)s"
+DEBUG_FMT = "[%(levelname)6s][%(asctime)s][%(name)15s]"\
+            "[%(module)s.%(funcName)s:%(lineno)4d]: %(message)s"
 
 
 def getLogger(name):
@@ -42,11 +42,16 @@ def log_level(level):
 
 
 # main jip logger configuration
-log = logging.getLogger('jip')
+log = logging.getLogger()
 log.handler = []
 log.propagate = False
 level = _get_level(getenv("JIP_LOGLEVEL", logging.WARN))
 log_level(level)
+if getenv("JIP_DB_LOGLEVEL", "") != "":
+    getLogger('sqlalchemy.engine').setLevel(
+        getenv("JIP_DB_LOGLEVEL", logging.INFO)
+    )
+
 
 console = logging.StreamHandler()
 fmt = DEFAULT_FMT
