@@ -3,6 +3,7 @@
 used to create pipeline graphs
 """
 import collections
+import os
 
 from jip.options import Option
 from jip.tools import Tool
@@ -28,6 +29,11 @@ class Job(Profile):
         self._node = None
         self._in_pipeline_name = None
 
+    @classmethod
+    def from_profile(cls, profile, pipeline):
+        job = cls(pipeline=pipeline, **(profile.__dict__))
+        return job
+
     def __getstate__(self):
         data = self.__dict__.copy()
         data['_pipeline'] = None
@@ -128,10 +134,12 @@ class Pipeline(object):
         self.excludes = []
         self._node_index = 0  # unique steadily increasing number
         self._utils = None
+        self._cwd = self._job.working_dir
 
     def __getstate__(self):
         data = {}
         data['_job'] = self._job
+        data['_cwd'] = self._cwd
         data['_current_job'] = self._current_job
         data['_name'] = self._name
         data['_node_index'] = self._node_index
@@ -140,6 +148,7 @@ class Pipeline(object):
 
     def __setstate__(self, data):
         ## update dict
+        self.__dict__['_cwd'] = data['_cwd']
         self.__dict__['_edges'] = set([])
         self.__dict__['_component_index'] = {}
         self.__dict__['_cleanup_nodes'] = []
@@ -250,6 +259,44 @@ class Pipeline(object):
         else:
             tool = _tool_name
         node = self.add(tool, _job=_job)
+
+        # add options if specified in kwargs
+        def _add_opts(option_type):
+            def _add(opts, name, kwargs=None):
+                kwargs = kwargs if kwargs else {}
+                if option_type == "_inputs":
+                    opts.add_input(name, **kwargs)
+                elif option_type == '_outputs':
+                    opts.add_output(name, **kwargs)
+                else:
+                    opts.add_option(name, **kwargs)
+
+            if option_type in kwargs:
+                for name, value in kwargs[option_type].iteritems():
+                    opts = node._tool.options
+                    if isinstance(value, dict):
+                        # get and remove any value set here,
+                        # otherwise this will influence the nargs
+                        # setting of the new option. We set the
+                        # value later anyways. We remove it from the
+                        # dict only if nargs is set. That means that
+                        # nargs will dominate
+                        v = None
+                        if "value" in value:
+                            v = value["value"]
+                            if "nargs" in value:
+                                del value["value"]
+                        _add(opts, name, value)
+                        if v is not None:
+                            node.set(name, v, allow_stream=False)
+                    else:
+                        _add(opts, name)
+                        node.set(name, value, allow_stream=False)
+                del kwargs[option_type]
+        _add_opts("_inputs")
+        _add_opts("_outputs")
+        _add_opts("_options")
+
         for k, v in kwargs.iteritems():
             node.set(k, v, allow_stream=False)
         return node
@@ -303,6 +350,9 @@ class Pipeline(object):
             return n
         elif not tool in self._nodes:
             n = Node(tool, self)
+            if not _job and tool._job:
+                # load profile from tool
+                _job = Job.from_profile(tool._job, self)
             # set the job
             job = _job() if _job else self._current_job()
             n._tool._job = job
@@ -317,6 +367,8 @@ class Pipeline(object):
             name = tool.name if not job.name else job.name
             log.debug("Add node | added %s", name)
             self._apply_node_name(n, name)
+            # set pipeline profile
+            n._pipeline_profile = _job() if _job else self._current_job()
         return self._nodes[tool]
 
     def _apply_node_name(self, node, name):
@@ -768,7 +820,14 @@ class Pipeline(object):
             #_render_nodes(self, list(self.nodes()))
 
             updated = set([])
+            cwd = self._cwd
+            if cwd is None:
+                cwd = os.getcwd()
+
             for node in self.topological_order():
+                # ensure a working directory is set
+                if node._job.working_dir is None:
+                    node._job.working_dir = cwd
                 node._tool.options.make_absolute(node._job.working_dir)
                 for link in [l for e in node.outgoing() for l in e._links]:
                     source = link[0]
@@ -810,6 +869,7 @@ class Pipeline(object):
             #done = set([])
             #for child in j.outgoing():
                 #transitive_reduction(j, child._target, done)
+
         log.info("Expand | Expansion finished. Nodes: %d", len(self))
         return fanout_done
 
@@ -908,7 +968,14 @@ class Pipeline(object):
             # are rendered properly and the values are set accordingly
             #if hasattr(node._tool, 'pipeline'):
             node._tool.setup()
+            # reapply the pipeline profile so it precedes the tool profile
+            if node._pipeline_profile:
+                node._pipeline_profile.update(node._job, overwrite=False)
+                node._job.update(node._pipeline_profile)
+            # make the nodes options absolute (Issue #38)
+            node._tool.options.make_absolute(node._job.working_dir)
             _render_nodes(self, [node])
+            node._tool.options.make_absolute(node._job.working_dir)
             sub_pipe = node._tool.pipeline()
             if sub_pipe is None:
                 continue
@@ -917,6 +984,7 @@ class Pipeline(object):
 
             # merge the nodes jobs with the sub-pipeline nodes
             for sub_node in sub_pipe.nodes():
+                node._job.apply_to_node(sub_node)
                 sub_node._job.merge(node._job)
 
             # render and apply the nodes name as pipeline
@@ -929,6 +997,11 @@ class Pipeline(object):
                 self.excludes.extend(sub_pipe.excludes)
             check_fanout = sub_pipe.expand(validate=validate, _find_dup=False,
                                            _check_fanout=check_fanout)
+
+            # apply pipeline profile
+            node._job.apply_to_pipeline(sub_pipe)
+            _render_jobs(sub_pipe, list(sub_pipe.nodes()))
+
             # find all nodes in the sub_pipeline
             # with no incoming edges and connect
             # them to the current nodes incoming nodes
@@ -970,6 +1043,10 @@ class Pipeline(object):
             self._expand_subpipe_resolve_incoming(node, sub_pipe)
             # setup the node and render values before we remove the node
             node._tool.setup()
+            # reapply the pipeline profile so it precedes the tool profile
+            if node._pipeline_profile:
+                node._pipeline_profile.update(node._job, overwrite=False)
+                node._job.update(node._pipeline_profile)
             _create_render_context(self, node._tool, node)
             #_render_nodes(self, [node])
 
@@ -1357,6 +1434,7 @@ class Node(object):
         self.__dict__['_graph'] = graph
         self.__dict__['_name'] = graph._name
         self.__dict__['_pipeline'] = graph._name
+        self.__dict__['_pipeline_profile'] = None
         self.__dict__['_index'] = index
         # the _node_index is an increasing counter that indicates
         # the order in which nodes were added to the pipeline graph
@@ -1411,7 +1489,7 @@ class Node(object):
         :param kwargs: option arguments for the tool
         :returns: tuple of (pipeline, node)
         """
-        pipeline = Pipeline()
+        pipeline = Pipeline(cwd=self._graph._cwd)
         job = self._graph.job()
         job._node = None
         job._pipeline = pipeline
@@ -1617,14 +1695,16 @@ class Node(object):
                     return link
         return None
 
-    def depends_on(self, other):
+    def depends_on(self, *args):
         """Add an explicit dependency between this node and the other
-        node.
+        node. The function accepts multiple values so you can specify multiple
+        parents at once.
 
-        :param other: the parent node
+        :param args*: all parent nodes.
         :type other: :class:`Node`
         """
-        self._graph.add_edge(other, self)
+        for other in args:
+            self._graph.add_edge(other, self)
 
     def group(self, other):
         """Groups this not and the other node. This creates a dependency
@@ -1823,7 +1903,8 @@ class Node(object):
 
     def __setattr__(self, name, value):
         if name in ["_job", "_index", "_pipeline",
-                    "_node_index", "_name", "_graph", "_edges", '_tool']:
+                    "_node_index", "_name", "_graph", "_edges", '_tool',
+                    '_pipeline_profile']:
             self.__dict__[name] = value
         else:
             self.set(name, value, allow_stream=False)
@@ -2084,6 +2165,36 @@ def _render_nodes(pipeline, nodes):
             _render_option(o, _create)
 
 
+def _render_jobs(pipeline, nodes):
+    # create the base dict that contains all ndoes
+    nds = {}
+    for n in pipeline.nodes():
+        nds[n.name] = n
+
+    # create a context for each node and set it for each option
+    ctxs = {}
+    for node in nodes:
+        ctx = _create_render_context(pipeline, node._tool, node, nds)
+        ctxs[node] = ctx
+
+    def _create(tool):
+        return _create_render_context(pipeline, tool, None, nds)
+
+    # render out all node options
+    for node in nodes:
+        # render working dir
+        ctx = ctxs[node]
+        # update name and job
+        ctx['name'] = node._job.name
+        ctx['job'] = node._job
+        if node._job.dir:
+            node._job.working_dir = render_template(node._job.dir, **ctx)
+        if node._job.out:
+            node._job.out = render_template(node._job.out, **ctx)
+        if node._job.err:
+            node._job.err = render_template(node._job.err, **ctx)
+
+
 def _render_option(option, create_fun):
     if option.render_context is None:
         create_fun(option.source)
