@@ -44,7 +44,7 @@ or pipeline:
             property that is also considered when you execute a pipeline or
             tool outside of a compute grid.
 
-        working_dir
+        working_dir or dir
             The working directory for a job. This is initialized to the
             current working directory of the process that creates the profile.
 
@@ -117,6 +117,7 @@ depends on the cluster implementation and the capabilities of the cluster:
 
 """
 import collections
+import fnmatch
 import re
 import os
 import json
@@ -138,8 +139,9 @@ class Profile(object):
                  tasks_per_node=None, environment=None, time=None, queue=None,
                  priority=None, log=None, out=None, account=None, mem=0,
                  extra=None, profile=None, prefix=None, temp=False, _load=True,
-                 env=None, tool_name=None, working_dir=None, description=None):
-        self._name = name  # render_template(name)
+                 env=None, tool_name=None, working_dir=None, description=None,
+                 specs=None, _name=None, **kwargs):
+        self._name = name if not _name else _name  # render_template(name)
         self.environment = render_template(environment)
         self.nodes = render_template(nodes)
         self.threads = render_template(threads)
@@ -150,21 +152,55 @@ class Profile(object):
         self.time = render_template(time)
         self.mem = render_template(mem)
         self.priority = render_template(priority)
-        self.log = render_template(log)
-        self.out = render_template(out)
+        self.log = log
+        self.out = out
         self.account = render_template(account)
         self.prefix = render_template(prefix)
         self.description = description
-        self.env = None
+        self.env = env
         self.temp = temp
         self.extra = extra
-        self.job_specs = None
         self.tool_name = tool_name
-        self.working_dir = working_dir if working_dir is not None \
-            else os.getcwd()
+        self.working_dir = working_dir
+        if self.working_dir is None and kwargs.get('dir', None):
+            self.working_dir = kwargs['dir']
+        self.specs = specs if specs else {}
         if profile is not None and _load:
             self.load(profile)
 
+    def apply_to_pipeline(self, pipeline):
+        """Apply this profile to the pipeline
+
+        :param pipeline: the pipeline
+        :type pipeline: :class:`jip.pipeline.Pipeline`
+        """
+        for node in pipeline.nodes():
+            self.apply_to_node(node)
+
+    def apply_to_node(self, node):
+        # check if there is a matching spec for the node
+        node_profile = self.specs.get(node.name, None)
+        if not node_profile:
+            node_profile = self.specs.get(node._name, None)
+        # check via regexp
+        for spec_name, spec in self.specs.iteritems():
+            if fnmatch.fnmatch(node.name, spec_name):
+            #if re.match(spec_name, node.name):
+                if not node_profile:
+                    node_profile = spec
+                else:
+                    node_profile.update(spec)
+
+        if node_profile:
+            node._job.update(node_profile)
+            if node._pipeline_profile:
+                node._pipeline_profile.update(node_profile)
+
+        # apply global profile, don't overwrite
+        node._job.update(self, overwrite=False)
+        if node._pipeline_profile:
+            node._pipeline_profile.update(self, overwrite=False)
+
     @property
     def err(self):
         """Set the jobs error log file
@@ -179,6 +215,20 @@ class Profile(object):
     def err(self, value):
         self.log = value
 
+    @property
+    def dir(self):
+        """Set the jobs working directory
+
+        :getter: access the jobs working directory
+        :setter: set the jobs working directory
+        :type: string
+        """
+        return self.working_dir
+
+    @dir.setter
+    def dir(self, value):
+        self.working_dir = value
+
     @property
     def name(self):
         """Set the jobs name
@@ -232,8 +282,16 @@ class Profile(object):
             k = re.sub("^-+", "", k)
             k = re.sub("-", "_", k)
             if v and hasattr(self, k):
-                setattr(self, k, v)
-        ## handle tasks per node explicitly
+                # check for multiple values
+                for single in v.split(" "):
+                    tup = single.split("=")
+                    if len(tup) == 1:
+                        setattr(self, k, single)
+                    else:
+                        # find or create a spec for the given key
+                        spec_profile = self.specs.get(tup[0], Profile())
+                        setattr(spec_profile, k, tup[1])
+                        self.specs[tup[0]] = spec_profile
 
     def _render_job_name(self, job):
         ctx = {}
@@ -252,27 +310,22 @@ class Profile(object):
         ctx = {}
         for o in job.tool.options:
             ctx[o.name] = o
+        ctx['name'] = self.name
+        ctx['job'] = self
         return render_template(
             "%s%s" % ("" if not self.prefix else self.prefix, name), **ctx
         )
 
-    def apply(self, job, _load_specs=True, overwrite_threads=False,
-              pipeline=False):
-        """Apply this profile to a given job and all its ambedded children
-        All non-None values are applied to the given job.
+    def apply_overwrite(self, job):
+        """Apply the profile and overwrite all settings that are set
+        in this profile
         """
-        log.debug("Profiles | Applying job profile to %s", job)
-        if not pipeline:
+        log.debug("Profiles | Overwriting job profile to %s", job)
+
+        if self.name:
             job.name = self._render_job_name(job)
-        elif self.name is not None:
-            log.info("Apply pipeline name to job: %s %s", job, self.name)
-            job.pipeline = self._render(job, self.name)
-        if self.threads is not None and job.threads < 1:
-            if not overwrite_threads:
-                job.threads = max(int(self.threads), job.threads)
-            else:
-                job.threads = int(self.threads)
-        # TODO: add the same override logic as for threads
+        if self.threads:
+            job.threads = int(self.threads)
         if self.nodes is not None:
             job.nodes = self.nodes
         if self.tasks is not None:
@@ -302,6 +355,12 @@ class Profile(object):
         if self.working_dir is not None:
             job.working_directory = os.path.abspath(self.working_dir)
 
+        # make log files absolute
+        if job.stdout and not job.stdout.startswith("/"):
+            job.stdout = os.path.join(job.working_directory, job.stdout)
+        if job.stderr and not job.stderr.startswith("/"):
+            job.stderr = os.path.join(job.working_directory, job.stderr)
+
         # load environment
         if self.env:
             current = os.environ.copy()
@@ -312,37 +371,101 @@ class Profile(object):
                 rendered[k] = render_template(v, **current)
             job.env.update(rendered)
 
-        if specs is None:
-            get_specs()
+        if hasattr(job, 'pipe_to'):
+            for child in job.pipe_to:
+                self.apply_overwrite(child)
+        # check specs
+        for spec_name, spec in self.specs.iteritems():
+            if fnmatch.fnmatch(job.name, spec_name):
+                spec.apply_overwrite(job)
 
-        if _load_specs:
-            if job.tool.name in specs:
-                # apply the job spec
-                spec_profile = Profile(threads=self.threads)
-                spec_profile.load_spec(specs[job.tool.name], None)
-                spec_profile.apply(job, False, overwrite_threads=True)
+    def apply(self, job, pipeline=False, overwrite=False):
+        """Apply this profile to the given job."""
+        log.debug("Profiles | Applying job profile to %s", job)
+        if overwrite:
+            self.apply_overwrite(job)
+            return
 
-            if self.tool_name in specs:
-                # apply the job spec
-                spec_profile = Profile(threads=self.threads)
-                spec_profile.load_spec(specs[self.tool_name], None)
-                spec_profile.apply(job, False)
-                spec = specs[self.tool_name]
-                if 'jobs' in spec and job.tool.name in spec['jobs']:
-                    spec_profile = Profile(threads=self.threads)
-                    spec_profile.load_spec(spec['jobs'][job.tool.name], None)
-                    spec_profile.apply(job, False, overwrite_threads=True)
+        # set the job name or the pipeline name
+        # if this is a job or a pipeline
+        if not pipeline:
+            job.name = self._render_job_name(job)
+        elif self.name is not None:
+            log.info("Apply pipeline name to job: %s %s", job, self.name)
+            job.pipeline = self._render(job, self.name)
 
-        if self.job_specs is not None and job.tool.name in self.job_specs:
-            # apply the job spec
-            spec_profile = Profile(threads=self.threads)
-            spec_profile.load_spec(self.job_specs[job.tool.name], None)
-            spec_profile.apply(job, False, overwrite_threads=True)
+        if self.threads and job.threads is None:
+            job.threads = int(self.threads)
+        if self.nodes is not None and job.nodes is None:
+            job.nodes = self.nodes
+        if self.tasks is not None and job.tasks is None:
+            job.tasks = self.tasks
+        if self.tasks_per_node is not None and job.tasts_per_node is None:
+            job.tasks_per_node = self.tasks_per_node
+        if self.environment is not None and job.environment is None:
+            job.environment = self.environment
+        if self.queue is not None and job.queue is None:
+            job.queue = self.queue
+        if self.priority is not None and job.priority is None:
+            job.priority = self.priority
+        if self.time is not None and job.max_time is None:
+            job.max_time = jip.utils.parse_time(self.time)
+        if self.mem is not None and job.max_memory is None:
+            job.max_memory = jip.utils.parse_mem(self.mem)
+        if self.log is not None and job.stderr is None:
+            job.stderr = self._render(job, self.log)
+        if self.out is not None and job.stdout is None:
+            job.stdout = self._render(job, self.out)
+        if self.account is not None and job.account is None:
+            job.account = self.account
+        if self.temp is not None and job.temp is None:
+            job.temp = self.temp
+        if self.extra is not None and job.extra is None:
+            job.extra = self.extra
+        if self.working_dir is not None and job.working_directory is None:
+            job.working_directory = os.path.abspath(self.working_dir)
+
+        # make log files absolute
+        if job.stdout and not job.stdout.startswith("/"):
+            job.stdout = os.path.join(job.working_directory, job.stdout)
+        if job.stderr and not job.stderr.startswith("/"):
+            job.stderr = os.path.join(job.working_directory, job.stderr)
+
+        # load environment
+        if self.env:
+            current = os.environ.copy()
+            if job.env:
+                current.update(job.env)
+            rendered = {}
+            for k, v in self.env.iteritems():
+                rendered[k] = render_template(v, **current)
+            job.env.update(rendered)
 
         if hasattr(job, 'pipe_to'):
             for child in job.pipe_to:
                 self.apply(child)
 
+    def update(self, profile, overwrite=True):
+        """Update this profile from a given profile. All values that are
+        not None in the other profile are applied to this
+        profile
+
+        :param profile: the other profile
+        :type profile: :class:`Profile`
+        :param overwrite: if True, value will be set regardless. Otherwise, the
+                          new value will only be applied if the old value
+                          is None
+        """
+        attrs = ["environment", "nodes", "threads",
+                 "tasks", "tasks_per_node", "queue",
+                 "time", "mem", "priority", "log", "out",
+                 "account", "prefix", "env", "temp", "extra", "working_dir"]
+        for attr in attrs:
+            other = profile.__getattribute__(attr)
+            if other is not None and (overwrite or
+                                      self.__getattribute__(attr) is None):
+                setattr(self, attr, other)
+
     def merge(self, master):
         """Merge this profile with the given master profile.
 
@@ -350,15 +473,15 @@ class Profile(object):
 
         :param master: the master profile
         """
-        self.working_dir = master.working_dir if master.working_dir \
+        self.working_dir = master.working_dir if self.working_dir is None\
             else self.working_dir
 
     def __call__(self, name=None, threads=None, nodes=None, tasks=None,
                  tasks_per_node=None, environment=None, time=None, queue=None,
-                 priority=None, log=None, out=None, account=None, mem=None,
-                 profile=None, prefix=None, temp=None, extra=None, dir=None,
-                 description=None):
-        return self.__class__(
+                 priority=None, log=None, out=None, err=None, account=None,
+                 mem=None, profile=None, prefix=None, temp=None, extra=None,
+                 dir=None, description=None, env=None):
+        clone = self.__class__(
             name=name if name is not None else self._name,
             threads=threads if threads is not None else self.threads,
             tasks=tasks if tasks is not None else self.tasks,
@@ -366,12 +489,14 @@ class Profile(object):
             self.tasks_per_node,
             environment=environment if environment is not None
             else self.environment,
+            env=env if env is not None else self.env,
             nodes=nodes if nodes is not None else self.nodes,
             profile=profile if profile is not None else self.profile,
             queue=queue if queue is not None else self.queue,
             time=time if time is not None else self.time,
             priority=priority if priority is not None else self.priority,
-            log=log if log is not None else self.log,
+            log=log if log is not None else
+            (err if err is not None else self.log),
             out=out if out is not None else self.out,
             account=account if account is not None else self.account,
             mem=mem if mem is not None else self.mem,
@@ -383,25 +508,13 @@ class Profile(object):
             else self.description,
             _load=False
         )
+        for name, spec in self.specs.iteritems():
+            clone.specs[name] = spec()
+        return clone
 
     def __repr__(self):
         return str(vars(self))
 
-    def load_spec(self, spec, tool):
-        """Update this profile from the specifications default parameters
-
-        :param spec: dictionary with the job specification
-        :param tool: name of the tool or pipeline
-        """
-        if spec is not None and (tool is None or tool in spec):
-            d = spec if tool is None or tool not in spec else spec[tool]
-            for k, v in d.iteritems():
-                if v is None:
-                    continue
-                self.__setattr__(k, v)
-            if tool is not None and 'jobs' in spec[tool]:
-                self.job_specs = spec[tool]['jobs']
-
     @classmethod
     def from_job(cls, job):
         """Create a profile based on a given job. All properties
@@ -429,12 +542,48 @@ class Profile(object):
         profile.env = job.env
         return profile
 
+    @classmethod
+    def from_file(cls, file_name):
+        """Load a profile from a json file
+
+        :param file_name: the name of the input file
+        """
+        with open(file_name) as of:
+            data = json.load(of)
+            return cls.from_dict(data)
+
+    @classmethod
+    def from_dict(cls, data):
+        """Load a profile from a dictionary"""
+        profile = cls()
+        # apply all the params
+        for k, v in data.iteritems():
+            if k != 'jobs':
+                profile.__setattr__(k, v)
+        if "jobs" in data:
+            for name, spec in data["jobs"].iteritems():
+                profile.specs[name] = cls.from_dict(spec)
+        return profile
+
 
 def get(name='default', tool=None):
     """Load a profile by name. If tools is speciefied, the specs are
     searched to the tool and if found, the spec is applied.
     """
-    p = Profile(profile=name)
+    # check the name for specs
+    s = name.split(' ')
+    p = Profile()
+    for ss in s:
+        tup = ss.split("=")
+        if len(tup) == 1:
+            # update global
+            l = Profile(profile=tup[0])
+            p.update(l)
+        else:
+            # update or create spec
+            spec = p.specs.get(tup[0], Profile())
+            spec.update(Profile(profile=tup[1]))
+            p.specs[tup[0]] = spec
     return p
 
 
