@@ -22,3 +22,17 @@ def test_resolve_id_ranges_number_exception():
 def test_resolve_id_ranges_negative_number_exception():
     with pytest.raises(ValueError):
         cli.resolve_job_range('-1')
+
+
+def test_parse_arg_lists():
+    docstring = """The tool
+
+Usage:
+    tool -o <out>...
+
+Options:
+    -o, --output <out>...   The list
+"""
+    args = cli.parse_args(docstring, ['-o', 'A', 'B'],
+                          options_first=False)
+    assert args['--output'] == ['A', 'B']
