@@ -1014,3 +1014,45 @@ def test_multiplex_with_stream():
     assert len(jobs) == 4
     execs = jip.create_executions(jobs)
     assert len(execs) == 2
+
+
+def test_depends_on_with_multiple_nodes():
+    p = jip.Pipeline()
+    a = p.bash('hostname')
+    b = p.bash('hostname')
+    c = p.bash('hostname')
+    a.depends_on(c, b)
+
+    assert len(list(a.incoming())) == 2
+
+
+def test_fanin_pipeline():
+    @jip.tool()
+    def merger():
+        """\
+        The merger
+
+        Usage:
+            merger -i <input>... -o <output>
+
+        Options:
+            -i, --input <input>...   The input files
+            -o, --output <output>    The output file
+        """
+        return "echo Merge"
+
+    p = jip.Pipeline()
+    align = p.bash('cat ${input}', input=["A", "B"], output="${input}.aln")
+    merge = p.run('merger', input=align, output='result')
+    jobs = jip.create_jobs(p, validate=False)
+    assert len(jobs) == 3
+
+
+def test_embedded_options_are_absolute():
+    jip.scanner.add_folder("test/data/makeabs")
+    p = jip.Pipeline()
+    p.run('makeabs', infile="Makefile", output="result")
+    jobs = jip.create_jobs(p)
+    assert len(jobs) == 1
+    cwd = os.getcwd()
+    assert jobs[0].command == "(cat %s/Makefile)> %s/result" % (cwd, cwd)
